/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

class CustomHeaderReaderTest {

	@InjectMocks
	private CustomHeaderReader reader;

	@Mock
	private Document document;
	@Mock
	private Element element;
	@Mock
	private Attr attribute;

	@BeforeEach
	void setup() {
		when(document.getDocumentElement()).thenReturn(element);
	}

	@Test
	void shouldNotAddIfNotPresent() {
		var formData = reader.getHeader(document);

		assertThat(formData).isEmpty();
	}

	@Nested
	class TestHeaderAdded {

		@BeforeEach
		void setup() {
			when(element.getAttributeNode(any())).thenReturn(attribute);
		}

		@Test
		void shouldAddPostfachId() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_POSTFACH_ID);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.POSTFACH_ID);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_POSTFACH_ID, CustomHeaderTestFactory.POSTFACH_ID);
		}

		@Test
		void shouldAddVorname() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_VORNAME);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.VORNAME);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_VORNAME, CustomHeaderTestFactory.VORNAME);
		}

		@Test
		void shouldAddNachname() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_NACHNAME);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.NACHNAME);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_NACHNAME, CustomHeaderTestFactory.NACHNAME);
		}

		@Test
		void shouldAddGeburtsname() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_GEBURTSNAME);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.GEBURTSNAME);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_GEBURTSNAME, CustomHeaderTestFactory.GEBURTSNAME);
		}

		@Test
		void shouldAddGeburtsort() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_GEBURTSORT);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.GEBURTSORT);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_GEBURTSORT, CustomHeaderTestFactory.GEBURTSORT);
		}

		@Test
		void shouldAddEmail() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_EMAIL);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.EMAIL);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_EMAIL, CustomHeaderTestFactory.EMAIL);
		}

		@Test
		void shouldAddTelefon() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_TELEFON);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.TELEFON);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_TELEFON, CustomHeaderTestFactory.TELEFON);
		}

		@Test
		void shouldAddStrasse() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_STRASSE);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.STRASSE);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_STRASSE, CustomHeaderTestFactory.STRASSE);
		}

		@Test
		void shouldAddPlz() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_PLZ);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.PLZ);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_PLZ, CustomHeaderTestFactory.PLZ);
		}

		@Test
		void shouldAddOrt() {
			when(attribute.getName()).thenReturn(CustomHeaderReader.HEADER_ORT);
			when(attribute.getValue()).thenReturn(CustomHeaderTestFactory.ORT);

			var formData = reader.getHeader(document);

			assertThat(formData).containsEntry(CustomHeaderReader.HEADER_ORT, CustomHeaderTestFactory.ORT);
		}
	}

}