package de.ozgcloud.eingang.semantik.enginebased.afm;

import static java.util.Objects.*;
import static org.apache.commons.lang3.StringUtils.*;

import java.util.Collections;
import java.util.Map;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;

@Component
public class AfmAntragstellerHeaderMapper {

	static final String KEY_POSTFACH_ID = "u:saml_legacypostkorbhandle";
	static final String KEY_VORNAME = "u:saml_givenname";
	static final String KEY_NACHNAME = "u:saml_surname";
	static final String KEY_GEBURTSORT = "u:saml_placeofbirth";
	public static final String KEY_GEBURTSNAME = "u:saml_birthname";
	static final String KEY_EMAIL = "u:saml_mail";
	static final String KEY_TELEFON = "u:saml_telephonenumber";
	static final String KEY_STRASSE = "u:saml_postaladdress";
	static final String KEY_PLZ = "u:saml_postalcode";
	static final String KEY_ORT = "u:saml_localityname";

	public FormData parseAntragstellerData(FormData formData) {
		return formData.toBuilder().antragsteller(buildAntragsteller(getHeaders(formData))).build();
	}

	Antragsteller buildAntragsteller(Map<String, Object> headers) {
		return Antragsteller.builder()
				.postfachId((String) headers.get(KEY_POSTFACH_ID))
				.vorname((String) headers.get(KEY_VORNAME))
				.nachname((String) headers.get(KEY_NACHNAME))
				.geburtsname((String) headers.get(KEY_GEBURTSNAME))
				.geburtsort((String) headers.get(KEY_GEBURTSORT))
				.email((String) headers.get(KEY_EMAIL))
				.telefon((String) headers.get(KEY_TELEFON))
				.strasse((String) headers.get(KEY_STRASSE))
				.plz((String) headers.get(KEY_PLZ))
				.ort((String) headers.get(KEY_ORT))
				.build();
	}

	public boolean isResponsible(FormData formData) {
		var headers = getHeaders(formData);
		return headers.containsKey(KEY_POSTFACH_ID) && isPostfachIdNotBlank(headers.get(KEY_POSTFACH_ID));
	}

	@SuppressWarnings("unchecked")
	Map<String, Object> getHeaders(FormData formData) {
		return (Map<String, Object>) formData.getFormData().getOrDefault(AfmHeaderMapper.HEADER_FIELD, Collections.emptyMap());
	}

	boolean isPostfachIdNotBlank(Object postfachId) {
		if (isNull(postfachId)) {
			return false;
		}
		if (postfachId instanceof String id) {
			return isNotBlank(id);
		}
		throw new TechnicalException("Unexpected type of postfach id: " + postfachId.getClass().getName());
	}
}
