/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.router.VorgangService;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;
import de.ozgcloud.eingang.semantik.formbased.FormBasedSemantikAdapter;

class SemantikAdapterTest {

	@InjectMocks
	private SemantikAdapter adapter;
	@Mock
	private EngineBasedSemantikAdapter engineAdapter;
	@Mock
	private FormBasedSemantikAdapter formAdapter;
	@Mock
	private VorgangService vorgangService;

	@Nested
	class TestProcessFormData {

		@Mock
		private FormData formData;
		@Mock
		private FormData engineAdapterResponse;
		@Mock
		private FormData formAdapterResponse;

		@BeforeEach
		void mockEngineAdapter() {
			when(engineAdapter.parseFormData(any())).thenReturn(engineAdapterResponse);
			when(formAdapter.parseFormData(any())).thenReturn(formAdapterResponse);
		}

		@Test
		void shouldCallEngineAdapter() {
			adapter.processFormData(formData);

			verify(engineAdapter).parseFormData(formData);
		}

		@Test
		void shouldCallFormAdapter() {
			adapter.processFormData(formData);

			verify(formAdapter).parseFormData(engineAdapterResponse);
		}

		@Test
		void shouldCallVorgangRemoteService() {
			adapter.processFormData(formData);

			verify(vorgangService).createVorgang(formAdapterResponse);
		}
	}
}